/**
 * @file Init api
 * @author sparklewhy@gmail.com
 */

'use strict';

import base from './base';
import {promisifyApis, interceptApis} from '../na/api';
import {getCurrApp} from '../na/index';

/**
 * Initialize promisify APIs and interception APIs.
 * Return false, if initialized.
 *
 * @return {boolean}
 */
export function initApis() {
    if (!this.__apisInited) {
        this.__apisInited = true;

        Object.assign(this, base);

        let promiseApis = this.$promisifyApis;
        let interceptAPis = this.$interceptApis;

        promisifyApis(promiseApis, this);
        interceptApis(interceptAPis, '$api', this);

        let reqApiInterceptOpts = interceptAPis && interceptAPis.request;
        if (reqApiInterceptOpts) {
            interceptApis({request: reqApiInterceptOpts}, '$http', this);
        }
        return true;
    }

    return false;
}

/**
 * extend props on Page and Component instance
 */
export function extendInstanceProps() {
    if (this.__isPropsExtended) {
        return;
    }

    this.__isPropsExtended = true;
    let propDescriptors = {
        $app: {
            get() {
                return getCurrApp();
            }
        }
    };
    Object.keys(base).forEach(k => {
        propDescriptors[k] = {
            get() {
                return base[k];
            }
        };
    });
    Object.defineProperties(this, propDescriptors);
}
